/*
**************************************************************************
                                 description
                             --------------------
    copyright            : (C) 2002 by Luis Carvalho
    email                : lpassos@mail.telepac.pt
**************************************************************************

**************************************************************************
*                                                                        *
*  This program is free software; you can redistribute it and/or modify  *
*  it under the terms of the GNU General Public License as published by  *
*  the Free Software Foundation; either version 2 of the License, or     *
*  (at your option) any later version.                                   *
*                                                                        *
**************************************************************************/


#include "pmlibraryobjectbrowser.h"

#include <qlayout.h>
#include <qlabel.h>
#include <qcombobox.h>

#include <kurl.h>
#include <klocale.h>
#include <kconfig.h>
#include <kmessagebox.h>
#include <kdialog.h>
#include <kfilepreview.h>
#include <kdiroperator.h>

#include <stdlib.h>

#include "pmlineedits.h"
#include "pmdialogeditbase.h"
#include "pmlibraryobject.h"
#include "pmlibraryobjectpreview.h"
#include "pmobjectdrag.h"

QSize PMLibraryObjectBrowser::s_size = QSize( 600, 400 );
QStringList PMLibraryObjectBrowser::s_libraryPaths;
PMLibraryObjectBrowser* PMLibraryObjectBrowser::s_pLibraryBrowser = NULL;

// ************** PMFileView **************************

PMFileView::PMFileView( QWidget* parent, const char* name )
         : KFileIconView( parent, name ) 
{ 
   setSelectionMode( KFile::Single );
}

void PMFileView::mousePressEvent( QMouseEvent* e )
{
   KFileIconView::mousePressEvent( e );

   if( e->button( ) == LeftButton )
   {
      m_pressed = true;
      m_pressedPos = e->pos( );
      return;
   }
}

void PMFileView::mouseReleaseEvent( QMouseEvent* e )
{
   KFileIconView::mouseReleaseEvent( e );

   m_pressed = false;
}

void PMFileView::mouseMoveEvent( QMouseEvent* e )
{
   if( m_pressed )
   {
      int x = e->pos( ).x( );
      int y = e->pos( ).y( );

      //Is it time to start a drag?
      if( abs( x - m_pressedPos.x( ) ) > KGlobalSettings::dndEventDelay( ) ||
          abs( y - m_pressedPos.y( ) ) > KGlobalSettings::dndEventDelay( ) )
      {
         emit dragStart( );
         m_pressed = false;
         return;
      }
   }
   else
     KFileIconView::mouseMoveEvent( e );
}

void PMFileView::startDrag( )
{
   if( KFileView::selectedItems( ) && !KFileView::selectedItems( )->isEmpty( ) )
   {
      KFileItem* file = KFileView::selectedItems( )->getFirst( );
   
      PMLibraryObject object( file->url( ) );
      if( object.objects( ) )
      {
         QStoredDrag* d = new QStoredDrag( "application/kpovmodeler", widget( ) );
         d->setEncodedData( *( object.objects( ) ) );
         d->dragCopy( );
         m_dragObject=d;
      }
   }
}

QDragObject* PMFileView::dragObject( ) const
{
   return m_dragObject;
}

// ************** PMLibraryObjectBrowser **************************

PMLibraryObjectBrowser::PMLibraryObjectBrowser( QWidget* parent, const char* name )
      : QWidget( parent, name )
{
   setCaption( i18n( "Library Objects" ) );
   resize( s_size );

   QVBoxLayout* vl = new QVBoxLayout( this, KDialog::spacingHint( ) );

   QHBoxLayout* hl = new QHBoxLayout( vl );
   QLabel* lbl = new QLabel( i18n( "Library: " ), this );
   m_pLibraryComboBox = new QComboBox( this );
   m_pLibraryComboBox->insertStringList( s_libraryPaths );
   m_pLibraryComboBox->setCurrentItem( 0 );
   m_pLibraryComboBox->setDuplicatesEnabled( false );
   hl->addWidget( lbl );
   hl->addWidget( m_pLibraryComboBox );
   hl->addStretch( 1 );

   m_pBrowser = new KDirOperator( KURL( ), this );
   m_pBrowser->setURL( KURL( m_pLibraryComboBox->currentText( ) ), true );
   m_pBrowser->setNameFilter( "*.kpml" );
   m_pBrowser->setShowHiddenFiles( true );
   vl->addWidget( m_pBrowser, 1 );

   // Set up the browser's view
   PMFileView* fileView = new PMFileView( m_pBrowser, "" );
   KFilePreview* filePreview = new KFilePreview( fileView, m_pBrowser, "" );
   filePreview->setPreviewWidget( new PMLibraryObjectPreview( m_pBrowser ), KURL( ) );
   m_pBrowser->setView( filePreview );

   // Clean up KDirOperator actions. KDirOperator provides too many actions for what we need.
   // We just want enough action to allow browsing of directories, nothing more.
   KActionCollection* coll = m_pBrowser->actionCollection( );
   KActionMenu* menu = ( KActionMenu* )( coll->action( "popupMenu" ) );
   menu->remove( coll->action( "properties" ) );
   menu->remove( coll->action( "view menu" ) );
   menu->remove( coll->action( "separate dirs" ) );
   menu->remove( coll->action( "delete" ) );
   menu->remove( coll->action( "sorting menu" ) );
   menu->remove( coll->action( "mkdir" ) );
   // The menu has 4 separators.
   menu->remove( coll->action( "separator" ) );
   menu->remove( coll->action( "separator" ) );
   menu->remove( coll->action( "separator" ) );
   menu->remove( coll->action( "separator" ) );

   // Connect all objects
   connect( m_pLibraryComboBox, SIGNAL( highlighted( const QString& ) ), 
                                SLOT( slotPathSelected( const QString& ) ) );
   connect( m_pBrowser, SIGNAL( dragStart( ) ), SLOT( slotDragStart( ) ) );
   connect( fileView, SIGNAL( dragStart( ) ), SLOT( slotDragStart( ) ) );

}

void PMLibraryObjectBrowser::setLibraryPaths( const QStringList& l )
{
   s_libraryPaths = l;
   if( s_pLibraryBrowser )
   {
      QString url = s_pLibraryBrowser->m_pLibraryComboBox->currentText( );
      int item = l.findIndex( url );
      s_pLibraryBrowser->m_pLibraryComboBox->clear( );
      s_pLibraryBrowser->m_pLibraryComboBox->insertStringList( l );
      s_pLibraryBrowser->m_pLibraryComboBox->setCurrentItem( item );
      if( url != s_pLibraryBrowser->m_pLibraryComboBox->currentText( ) )
         s_pLibraryBrowser->slotPathSelected( s_pLibraryBrowser->m_pLibraryComboBox->currentText( ) );
   }
}

void PMLibraryObjectBrowser::saveConfig( KConfig* cfg )
{
   cfg->setGroup( "Appearance" );
   cfg->writeEntry( "LibraryObjectBrowserSize", s_size );

   cfg->setGroup( "ObjectLibraries" );
   cfg->writeEntry( "LibraryPaths", s_libraryPaths );
}

void PMLibraryObjectBrowser::restoreConfig( KConfig* cfg )
{
   cfg->setGroup( "Appearance" );

   QSize defaultSize( 300, 200 );
   s_size = cfg->readSizeEntry( "LibraryObjectBrowserSize", &defaultSize );

   cfg->setGroup( "ObjectLibraries" );
   s_libraryPaths = cfg->readListEntry( "LibraryPaths" );
}

void PMLibraryObjectBrowser::resizeEvent( QResizeEvent* ev )
{
   s_size = ev->size( );
}

void PMLibraryObjectBrowser::slotDragStart( )
{
   if( m_pBrowser->selectedItems( ) && !m_pBrowser->selectedItems( )->isEmpty( ) )
   {
      KFileItem* file = m_pBrowser->selectedItems( )->getFirst( );

      PMLibraryObject object( file->url( ) );
      if( object.objects( ) )
      {
         QStoredDrag* d = new QStoredDrag( "application/kpovmodeler", this );
         d->setEncodedData( *( object.objects( ) ) );
         d->dragCopy( );
      }
   }
}

void PMLibraryObjectBrowser::showBrowser( )
{
   if( !s_pLibraryBrowser )
      s_pLibraryBrowser = new PMLibraryObjectBrowser( NULL );

   s_pLibraryBrowser->show( );
}

void PMLibraryObjectBrowser::slotPathSelected( const QString& str )
{
   m_pBrowser->setURL( KURL( str ), true );
}

#include "pmlibraryobjectbrowser.moc"
